'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var backendCommon = require('@backstage/backend-common');
var express = require('express');
var Router = require('express-promise-router');
var expressGraphql = require('express-graphql');
var graphql = require('graphql');
var core = require('@envelop/core');
var graphqlModules$1 = require('@envelop/graphql-modules');
var graphqlModules = require('graphql-modules');
var schema = require('@graphql-tools/schema');
var DataLoader = require('dataloader');
var catalogModel = require('@backstage/catalog-model');
var loadFiles = require('@graphql-tools/load-files');
var GraphQLJSON = require('graphql-type-json');
var utils = require('@graphql-tools/utils');
var lodash = require('lodash');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var express__default = /*#__PURE__*/_interopDefaultLegacy(express);
var Router__default = /*#__PURE__*/_interopDefaultLegacy(Router);
var DataLoader__default = /*#__PURE__*/_interopDefaultLegacy(DataLoader);
var GraphQLJSON__default = /*#__PURE__*/_interopDefaultLegacy(GraphQLJSON);

function createLoader({ catalog }) {
  return new DataLoader__default["default"](function fetch(refs) {
    return Promise.all(refs.map(async (ref) => {
      let entity = await catalog.getEntityByRef(ref);
      return entity != null ? entity : new core.EnvelopError(`no such node with ref: '${ref}'`);
    }));
  });
}

async function resolveType({ id }, { loader }) {
  const entity = await loader.load(id);
  if (!entity)
    return null;
  return entity.kind;
}

const Catalog = graphqlModules.createModule({
  id: "catalog",
  typeDefs: loadFiles.loadFilesSync(backendCommon.resolvePackagePath("@frontside/backstage-plugin-graphql", "src/app/modules/catalog/catalog.graphql")),
  resolvers: {
    Entity: { __resolveType: resolveType },
    Owner: { __resolveType: resolveType },
    Ownable: { __resolveType: resolveType },
    Dependency: { __resolveType: resolveType },
    Lifecycle: {
      EXPERIMENTAL: "experimental",
      PRODUCTION: "production",
      DEPRECATED: "deprecated"
    },
    JSON: GraphQLJSON__default["default"],
    JSONObject: GraphQLJSON.GraphQLJSONObject,
    Query: {
      entity: (_, { name, kind, namespace = "default" }) => ({ id: catalogModel.stringifyEntityRef({ name, kind, namespace }) })
    }
  }
});

const Core = graphqlModules.createModule({
  id: "core",
  typeDefs: loadFiles.loadFilesSync(backendCommon.resolvePackagePath("@frontside/backstage-plugin-graphql", "src/app/modules/core/core.graphql")),
  resolvers: {
    Node: {
      __resolveType: resolveType,
      id: async ({ id }, _, { loader }) => {
        const entity = await loader.load(id);
        if (!entity)
          return null;
        return id;
      }
    },
    Query: {
      node: (_, { id }) => ({ id })
    }
  }
});

const resolveMappers = [
  (objectField, interfaceField, schema) => {
    var _a;
    const fieldDirective = (_a = utils.getDirective(schema, interfaceField != null ? interfaceField : objectField, "field")) == null ? void 0 : _a[0];
    if (!fieldDirective)
      return;
    const fieldType = (interfaceField != null ? interfaceField : objectField).type;
    const isKeyValuePairs = fieldType instanceof graphql.GraphQLList && fieldType.ofType instanceof graphql.GraphQLObjectType && fieldType.ofType.name === "KeyValuePair";
    objectField.resolve = async ({ id }, _, { loader }) => {
      const entity = await loader.load(id);
      if (!entity)
        return null;
      const fieldValue = lodash.get(entity, fieldDirective.at);
      return isKeyValuePairs && fieldValue ? Object.entries(fieldValue).map(([key, value]) => ({ key, value })) : fieldValue;
    };
  },
  (objectField, interfaceField, schema) => {
    var _a;
    const fieldDirective = (_a = utils.getDirective(schema, interfaceField != null ? interfaceField : objectField, "relation")) == null ? void 0 : _a[0];
    if (!fieldDirective)
      return;
    const fieldType = (interfaceField != null ? interfaceField : objectField).type;
    const isList = fieldType instanceof graphql.GraphQLList || fieldType instanceof graphql.GraphQLNonNull && fieldType.ofType instanceof graphql.GraphQLList;
    objectField.resolve = async ({ id }, _, { loader }) => {
      var _a2, _b, _c;
      const entities = (_c = (_b = (_a2 = await loader.load(id)) == null ? void 0 : _a2.relations) == null ? void 0 : _b.filter(({ type, targetRef }) => {
        var _a3;
        const { kind } = catalogModel.parseEntityRef(targetRef);
        return type === ((_a3 = fieldDirective.type) != null ? _a3 : objectField.name) && (fieldDirective.kind ? kind === fieldDirective.kind : true);
      }).map(({ targetRef }) => ({ id: targetRef }))) != null ? _c : [];
      const [entity = null] = entities;
      return isList ? entities : entity;
    };
  }
];
const objectMapper = {
  [utils.MapperKind.OBJECT_TYPE]: (objectType, schema) => {
    const interfaces = traverseExtends(objectType, schema);
    const typeConfig = objectType.toConfig();
    const fieldsConfig = {
      ...interfaces.reduce((fields, interfaceType) => ({
        ...fields,
        ...interfaceType.toConfig().fields
      }), {}),
      ...typeConfig.fields
    };
    const newObjectType = new graphql.GraphQLObjectType({
      ...typeConfig,
      fields: fieldsConfig,
      interfaces
    });
    const interfaceFields = interfaces.reduce((fields, interfaceType) => ({
      ...fields,
      ...interfaceType.getFields()
    }), {});
    Object.entries(newObjectType.getFields()).forEach(([fieldName, fieldType]) => resolveMappers.forEach((mapper) => mapper(fieldType, interfaceFields[fieldName], schema)));
    return newObjectType;
  }
};
function traverseExtends(type, schema) {
  var _a;
  const extendDirective = (_a = utils.getDirective(schema, type, "extend")) == null ? void 0 : _a[0];
  const interfaces = [];
  if (extendDirective) {
    const extendType = schema.getType(extendDirective.type);
    if (!graphql.isInterfaceType(extendType)) {
      throw new Error(`The interface "${extendDirective.type}" described in @extend directive for "${type.name}" isn't abstract type or doesn't exist`);
    }
    interfaces.push(...traverseExtends(extendType, schema));
  }
  return graphql.isInterfaceType(type) ? [...interfaces, type] : interfaces;
}
const transform = (schema) => {
  return utils.mapSchema(schema, objectMapper);
};

function createGraphQLApp(options) {
  const application = create(options);
  const loader = createLoader(options);
  const run = core.envelop({
    plugins: [
      core.useExtendContext(() => ({ catalog: options.catalog, loader })),
      graphqlModules$1.useGraphQLModules(application)
    ]
  });
  return { run, application };
}
function create(options) {
  return graphqlModules.createApplication({
    schemaBuilder: ({ typeDefs, resolvers }) => transform(schema.makeExecutableSchema({
      typeDefs,
      resolvers
    })),
    modules: [Core, Catalog, ...options.modules]
  });
}

async function createRouter(options) {
  var _a;
  const { logger } = options;
  const { run, application } = createGraphQLApp({
    catalog: options.catalog,
    modules: (_a = options.modules) != null ? _a : []
  });
  const router = Router__default["default"]();
  router.use(express__default["default"].json());
  router.use((_, res, next) => {
    res.setHeader("Content-Security-Policy", "'self' http: 'unsafe-inline'");
    next();
  });
  router.get("/schema", (_, response) => {
    response.send(graphql.printSchema(application.schema));
  });
  router.use("/", expressGraphql.graphqlHTTP(async () => {
    const { parse, validate, contextFactory, execute } = run();
    return {
      schema: application.schema,
      graphiql: true,
      customParseFn: parse,
      customValidateFn: validate,
      customExecuteFn: async (args) => {
        return execute({
          ...args,
          contextValue: await contextFactory()
        });
      }
    };
  }));
  router.get("/health", (_, response) => {
    logger.info("PONG!");
    response.send({ status: "ok" });
  });
  router.use(backendCommon.errorHandler());
  return router;
}

exports.createGraphQLApp = createGraphQLApp;
exports.createRouter = createRouter;
//# sourceMappingURL=index.cjs.js.map
